using System;
using System.Collections;
using Server;
using Server.Items;
using Mat = Server.Engines.BulkOrders.BulkMaterialType;

namespace Server.Engines.BulkOrders
{
	[TypeAlias( "Scripts.Engines.BulkOrders.LargeCarpenterBOD" )]
	public class LargeCarpenterBOD : LargeBOD
	{
		public static double[] m_CarpenterMaterialChances = new double[]
			{
				0.501953125, // None
				0.250000000, // Pine
				0.125000000, // Walnut
				0.062500000, // Maple
				0.031250000, // Mahogany
				0.015625000, // Ash
				0.007812500, // Cedar
				0.003906250, // Sycamore
				0.001953125, // Cherry
                                0.001853125  // Yew
			};

		public override int ComputeFame()
		{
			int bonus = 0;

			if ( RequireExceptional )
				bonus += 50;

			if ( Material >= BulkMaterialType.DullCopper && Material <= BulkMaterialType.Valorite )
				bonus += 50 * (1 + (int)(Material - BulkMaterialType.DullCopper));
			else if ( Material >= BulkMaterialType.Spined && Material <= BulkMaterialType.Barbed )
				bonus += 100 * (1 + (int)(Material - BulkMaterialType.Spined));
                        else if ( Material >= BulkMaterialType.Pine && Material <= BulkMaterialType.Yew )
				bonus += 50 * (1 + (int)(Material - BulkMaterialType.Pine));

			return 100 + Utility.Random( bonus );
		}

		private static int[][][] m_GoldTable = new int[][][]
			{
				
				new int[][] // 5-Part Staff (regular)
				{
					new int[]{ 4000, 0, 0, 0, 0, 0, 0, 0, 0 },
					new int[]{ 6000, 0, 0, 0, 0, 0, 0, 0, 0 },
					new int[]{ 8000, 0, 0, 0, 0, 0, 0, 0, 0 }
				},
				new int[][] // 5-Part Staff (exceptional)
				{
					new int[]{ 7500, 0, 0, 0, 0, 0, 0, 0, 0 },
					new int[]{ 11250, 0, 0, 0, 0, 0, 0, 0, 0 },
					new int[]{ 15000, 0, 0, 0, 0, 0, 0, 0, 0 }
				},

			};

		public override int ComputeGold()
		{
			Type primaryType;

			if ( Entries.Length > 0 )
				primaryType = Entries[0].Details.Type;
			else
				return 0;
 
                        bool isStaff = ( primaryType == typeof( QuarterStaff ) );

			int index;

			if ( isStaff )
				index = 1;
			else
				index = 0;

			index *= 2;

			if ( RequireExceptional )
				++index;

			if ( index < m_GoldTable.Length )
			{
				int[][] table = m_GoldTable[index];

				if ( AmountMax >= 20 )
					index = 2;
				else if ( AmountMax >= 15 )
					index = 1;
				else
					index = 0;

				if ( index < table.Length )
				{
					int[] list = table[index];

					if ( Material >= BulkMaterialType.Pine && Material <= BulkMaterialType.Yew )
						index = (1 + (int)(Material - BulkMaterialType.Pine));
					else
						index = 0;

					if ( index < list.Length )
						return list[index];
				}
			}

			return 0;
		}

		[Constructable]
		public LargeCarpenterBOD()
		{
			LargeBulkEntry[] entries;
			bool useMaterials = true;

			switch ( Utility.Random( 1 ) )
			{
				default:
				case  0: entries = LargeBulkEntry.ConvertEntries( this, LargeBulkEntry.LargeStaff );  break;
				
			}

			int hue = 1630;
			int amountMax = Utility.RandomList( 10, 15, 20, 20 );
			bool reqExceptional = ( 0.825 > Utility.RandomDouble() );

			BulkMaterialType material;

			if ( useMaterials )
				material = GetRandomMaterial( BulkMaterialType.Pine, m_CarpenterMaterialChances );
			else
				material = BulkMaterialType.None;

			this.Hue = hue;
			this.AmountMax = amountMax;
			this.Entries = entries;
			this.RequireExceptional = reqExceptional;
			this.Material = material;
		}

		public LargeCarpenterBOD( int amountMax, bool reqExceptional, BulkMaterialType mat, LargeBulkEntry[] entries )
		{
			this.Hue = 1630;
			this.AmountMax = amountMax;
			this.Entries = entries;
			this.RequireExceptional = reqExceptional;
			this.Material = mat;
		}

		public int High( Mat mat )
		{
			return 2 + (3 * (int)mat);
		}

		public int Low( Mat mat )
		{
			return 3 * (int)mat;
		}

		public bool Check( int yew )
		{
			int ours = ( AmountMax < 20 ? Low( Material ) : High( Material ) );

			return ( ours == yew );
		}

		public bool Check( int min, int max )
		{
			int ours = ( AmountMax < 20 ? Low( Material ) : High( Material ) );

			return ( ours >= min && ours <= max );
		}

		private bool Check( Mat yew )
		{
			return ( Material == yew );
		}

		private bool Check( Mat start, Mat end )
		{
			return ( Material >= start && Material <= end );
		}

		public override ArrayList ComputeRewards()
		{
			Type primaryType;

			if ( Entries.Length > 0 )
				primaryType = Entries[0].Details.Type;
			else
				return new ArrayList();

			bool glovesPlusThree = false, glovesPlusFive = false;
			bool lumberjackaxe = false;
			bool stainofdurability = false;
			bool ps5 = false, ps10 = false, ps15 = false, ps20 = false;
			bool rPine = false, rWalnut = false, rMaple = false, rMahogany = false;
			bool rAsh = false, rCedar = false, rSycamore = false, rCherry = false, rYew = false;
			
			bool isBow = ( primaryType == typeof( Bow ) );

			if ( isBow )
			{
				if ( RequireExceptional )
				{
					ps5 = Check( Low( Mat.None ) );
					ps10 = Check( High( Mat.None ) );
					ps20 = Check( Low( Mat.Pine ), Low( Mat.Walnut ) );
					rPine = Check( Low( Mat.None ) );
					rWalnut = Check( Low( Mat.None ) );
					rMaple = Check( High( Mat.None ) );
					rMahogany = Check( Low( Mat.Pine ), Low( Mat.Walnut ) );
					rAsh = Check( High( Mat.Walnut ), Low( Mat.Mahogany ) );
					rCedar = Check( High( Mat.Mahogany ), Low( Mat.Cedar ) );
					rSycamore = Check( High( Mat.Cedar ), High( Mat.Cherry ) );
					rCherry = Check( High( Mat.Cedar ), High( Mat.Cherry ) );
                                        rYew = Check( High( Mat.Cherry ), High( Mat.Yew ) );
					
				}
				else
				{
					lumberjackaxe = Check( Mat.None );
					stainofdurability = Check( Mat.None );
					ps5 = Check( Mat.Pine );
					ps10 = Check( Mat.Pine, Mat.Walnut );
					ps15 = Check( Mat.Walnut, Mat.Mahogany );
					ps20 = Check( Mat.Mahogany, Mat.Cedar );
					rPine = Check( Mat.Pine );
					rWalnut = Check( Mat.Pine );
					rMaple = Check( Mat.Pine, Mat.Walnut );
					rMahogany = Check( Mat.Walnut, Mat.Cedar );
					rAsh = Check( Mat.Cedar, Mat.Sycamore );
					rCedar = Check( Mat.Cherry );
					
				}
			}
			
			ArrayList list = new ArrayList();

                        if ( glovesPlusThree )
				list.Add( new StuddedGlovesOfLumberjacking( 3 ) );

			if ( glovesPlusFive )
				list.Add( new RingmailGlovesOfLumberjacking( 5 ) );

			if ( lumberjackaxe )
				list.Add( new LumberjackAxe() );

			if ( stainofdurability )
				list.Add( new StainOfDurability() );

			if ( ps5 )
				list.Add( new PowerScroll( SkillName.Carpentry, 105 ) );

			if ( ps10 )
				list.Add( new PowerScroll( SkillName.Carpentry, 110 ) );

			if ( ps15 )
				list.Add( new PowerScroll( SkillName.Carpentry, 115 ) );

			if ( ps20 )
				list.Add( new PowerScroll( SkillName.Carpentry, 120 ) );

			if ( rPine )
				list.Add( new RunicSaw( CraftResource.Pine, Core.AOS ? 50 : 50 ) );

			if ( rWalnut )
				list.Add( new RunicSaw( CraftResource.Walnut, Core.AOS ? 45 : 50 ) );

			if ( rMaple )
				list.Add( new RunicSaw( CraftResource.Maple, Core.AOS ? 40 : 50 ) );

			if ( rMahogany )
				list.Add( new RunicSaw( CraftResource.Mahogany, Core.AOS ? 35 : 50 ) );

			if ( rAsh )
				list.Add( new RunicSaw( CraftResource.Ash, Core.AOS ? 30 : 50 ) );

			if ( rCedar )
				list.Add( new RunicSaw( CraftResource.Cedar, Core.AOS ? 25 : 50 ) );

			if ( rSycamore )
				list.Add( new RunicSaw( CraftResource.Sycamore, Core.AOS ? 20 : 50 ) );

			if ( rCherry )
				list.Add( new RunicSaw( CraftResource.Cherry, Core.AOS ? 15 : 50 ) );

                        if ( rYew )
				list.Add( new RunicSaw( CraftResource.Yew, Core.AOS ? 10 : 50 ) );
			
			return list;
		}

		public LargeCarpenterBOD( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}
}