using System;
using System.Collections;
using Server;
using Server.Items;
using Server.Engines.Craft;
using Mat = Server.Engines.BulkOrders.BulkMaterialType;

namespace Server.Engines.BulkOrders
{
	[TypeAlias( "Scripts.Engines.BulkOrders.SmallFletcherBOD" )]
	public class SmallFletcherBOD : SmallBOD
	{
		public static double[] m_FletcherMaterialChances = new double[]
			{
				0.501953125, // None
				0.250000000, // Pine
				0.125000000, // Walnut
				0.062500000, // Maple
				0.031250000, // Mahogany
				0.015625000, // Ash
				0.007812500, // Cedar
				0.003906250, // Sycamore
				0.001953125, // Cherry
                                0.001853125  // Yew
			};

		public int High( Mat mat )
		{
			return 2 + (3 * (int)mat);
		}

		public int Low( Mat mat )
		{
			return 3 * (int)mat;
		}

		public bool Check( int yew )
		{
			int ours = ( AmountMax < 20 ? Low( Material ) : High( Material ) );

			return ( ours == yew );
		}

		public bool Check( int min, int max )
		{
			int ours = ( AmountMax < 20 ? Low( Material ) : High( Material ) );

			return ( ours >= min && ours <= max );
		}

		private bool Check( Mat yew )
		{
			return ( Material == yew );
		}

		private bool Check( Mat start, Mat end )
		{
			return ( Material >= start && Material <= end );
		}

		public override int ComputeFame()
		{
			int bonus = 0;

			if ( RequireExceptional )
				bonus += 20;

			if ( Material >= BulkMaterialType.Pine && Material <= BulkMaterialType.Yew )
				bonus += 20 * (1 + (int)(Material - BulkMaterialType.Pine));
			
			return 10 + Utility.Random( bonus );
		}

		public override int ComputeGold()
		{
			int bonus = 0;

			if ( RequireExceptional )
				bonus += 500;

			if ( Material >= BulkMaterialType.Pine && Material <= BulkMaterialType.Yew )
				bonus += 500 * (1 + (int)(Material - BulkMaterialType.Pine));

			return 750 + Utility.Random( bonus );
		}

		public override ArrayList ComputeRewards()
		{
			if ( Type == null )
				return new ArrayList();

                        bool sturdyTool = false;
			bool glovesPlusThree = false, glovesPlusFive = false;
			bool lumberjackaxe = false;
			bool stainofdurability = false;
			bool ps5 = true, ps10 = true, ps15 = false, ps20 = false;
			bool rPine = true, rWalnut = true, rMaple = true, rMahogany = true;
			
			if ( Type.IsSubclassOf( typeof( BaseWeapon ) ) )
			{
				if ( RequireExceptional )
				{
					glovesPlusThree = true;
					lumberjackaxe = true;
				}
				else
				{
					sturdyTool = true;
				}
				
			}
			else
			{
				if ( RequireExceptional )
				{
                                        glovesPlusThree = Check( Mat.None ) || Check( High( Mat.Walnut ), Low( Mat.Maple ) );
					glovesPlusFive = Check( High( Mat.Walnut ), Low( Mat.Mahogany ) );
					lumberjackaxe = Check( Low( Mat.None ), Low( Mat.Walnut ) );
					stainofdurability = Check( Low( Mat.Pine ), High( Mat.Walnut ) );
					ps5 = Check( High( Mat.Mahogany ), Low( Mat.Ash ) );
					ps10 = Check( High( Mat.Ash ), Low( Mat.Cedar ) );
					ps15 = Check( High( Mat.Cedar ), Low( Mat.Cherry ) );
					ps20 = Check( High( Mat.Cherry ) );
					rPine = Check( High( Mat.Walnut ), Low( Mat.Ash ) );
					rWalnut = Check( High( Mat.Maple ), Low( Mat.Ash ) );
					rMaple = Check( High( Mat.Ash ), Low( Mat.Cedar ) );
					rMahogany = Check( High( Mat.Cedar ), High( Mat.Cherry ) );
				}
				else
				{
                                        sturdyTool = Check( Mat.None );
					glovesPlusThree = Check( Mat.Pine, Mat.Mahogany ) || Check( Mat.Cedar, Mat.Sycamore );
					glovesPlusFive = Check( Mat.Cedar, Mat.Cherry );
					lumberjackaxe = Check( Mat.Pine, Mat.Cedar );
					stainofdurability = Check( Mat.Walnut, Mat.Cedar );
					ps5 = Check( Mat.Cherry );
					rPine = Check( Mat.Cedar, Mat.Cherry );
					rWalnut = Check( Mat.Sycamore, Mat.Cherry );
				}
			}

			ArrayList list = new ArrayList();

                        if ( sturdyTool )
			{
				if ( Utility.RandomBool() )
					list.Add( new SturdyAxe() );
				
			}

			if ( glovesPlusThree )
				list.Add( new StuddedGlovesOfLumberjacking( 3 ) );

			if ( glovesPlusFive )
				list.Add( new RingmailGlovesOfLumberjacking( 5 ) );

			if ( lumberjackaxe )
				list.Add( new LumberjackAxe() );

                        if ( stainofdurability )
				list.Add( new StainOfDurability() );
			
			if ( ps5 )
				list.Add( new PowerScroll( SkillName.Fletching, 105 ) );

			if ( ps10 )
				list.Add( new PowerScroll( SkillName.Fletching, 110 ) );

			if ( ps15 )
				list.Add( new PowerScroll( SkillName.Fletching, 115 ) );

			if ( ps20 )
				list.Add( new PowerScroll( SkillName.Fletching, 120 ) );

			if ( rPine )
				list.Add( new RunicFletchersTools( CraftResource.Pine, Core.AOS ? 50 : 50 ) );

			if ( rWalnut )
				list.Add( new RunicFletchersTools( CraftResource.Walnut, Core.AOS ? 45 : 50 ) );

			if ( rMaple )
				list.Add( new RunicFletchersTools( CraftResource.Maple, Core.AOS ? 40 : 50 ) );

			if ( rMahogany )
				list.Add( new RunicFletchersTools( CraftResource.Mahogany, Core.AOS ? 35 : 50 ) );

			return list;
		}

		public static SmallFletcherBOD CreateRandomFor( Mobile m )
		{
			SmallBulkEntry[] entries;
			bool useMaterials;

			if ( useMaterials = Utility.RandomBool() )
				entries = SmallBulkEntry.BowFletchingWeapons;
			else
				entries = SmallBulkEntry.BowFletchingWeapons;

			if ( entries.Length > 0 )
			{
				double theirSkill = m.Skills[SkillName.Fletching].Base;
				int amountMax;

				if ( theirSkill >= 70.1 )
					amountMax = Utility.RandomList( 10, 15, 20, 20 );
				else if ( theirSkill >= 50.1 )
					amountMax = Utility.RandomList( 10, 15, 15, 20 );
				else
					amountMax = Utility.RandomList( 10, 10, 15, 20 );

				BulkMaterialType material = BulkMaterialType.None;

				if ( useMaterials && theirSkill >= 70.1 )
				{
					for ( int i = 0; i < 20; ++i )
					{
						BulkMaterialType check = GetRandomMaterial( BulkMaterialType.Pine, m_FletcherMaterialChances );
						double skillReq = 0.0;

						switch ( check )
						{
							case BulkMaterialType.Pine: skillReq = 65.0; break;
							case BulkMaterialType.Walnut: skillReq = 70.0; break;
							case BulkMaterialType.Maple: skillReq = 75.0; break;
							case BulkMaterialType.Mahogany: skillReq = 80.0; break;
							case BulkMaterialType.Ash: skillReq = 85.0; break;
							case BulkMaterialType.Cedar: skillReq = 90.0; break;
							case BulkMaterialType.Sycamore: skillReq = 95.0; break;
							case BulkMaterialType.Cherry: skillReq = 100.0; break;
                                                        case BulkMaterialType.Yew: skillReq = 110.0; break;
						}

						if ( theirSkill >= skillReq )
						{
							material = check;
							break;
						}
					}
				}

				double excChance = 0.0;

				if ( theirSkill >= 70.1 )
					excChance = (theirSkill + 80.0) / 200.0;

				bool reqExceptional = ( excChance > Utility.RandomDouble() );

				SmallBulkEntry entry = null;

				CraftSystem system = DefBowFletching.CraftSystem;

				for ( int i = 0; i < 150; ++i )
				{
					SmallBulkEntry check = entries[Utility.Random( entries.Length )];

					CraftItem item = system.CraftItems.SearchFor( check.Type );

					if ( item != null )
					{
						bool allRequiredSkills = true;
						double chance = item.GetSuccessChance( m, null, system, false, ref allRequiredSkills );

						if ( allRequiredSkills && chance >= 0.0 )
						{
							if ( reqExceptional )
								chance = item.GetExceptionalChance( system, chance, m );

							if ( chance > 0.0 )
							{
								entry = check;
								break;
							}
						}
					}
				}

				if ( entry != null )
					return new SmallFletcherBOD( entry, material, amountMax, reqExceptional );
			}

			return null;
		}

		private SmallFletcherBOD( SmallBulkEntry entry, BulkMaterialType material, int amountMax, bool reqExceptional )
		{
			this.Hue = 2213;
			this.AmountMax = amountMax;
			this.Type = entry.Type;
			this.Number = entry.Number;
			this.Graphic = entry.Graphic;
			this.RequireExceptional = reqExceptional;
			this.Material = material;
		}

		[Constructable]
		public SmallFletcherBOD()
		{
			SmallBulkEntry[] entries;
			bool useMaterials;

			if ( useMaterials = Utility.RandomBool() )
				entries = SmallBulkEntry.BowFletchingWeapons;
			else
				entries = SmallBulkEntry.BowFletchingWeapons;

			if ( entries.Length > 0 )
			{
				int hue = 2213;
				int amountMax = Utility.RandomList( 10, 15, 20 );

				BulkMaterialType material;

				if ( useMaterials )
					material = GetRandomMaterial( BulkMaterialType.Pine, m_FletcherMaterialChances );
				else
					material = BulkMaterialType.None;

				bool reqExceptional = Utility.RandomBool() || (material == BulkMaterialType.None);

				SmallBulkEntry entry = entries[Utility.Random( entries.Length )];

				this.Hue = 2213;
				this.AmountMax = amountMax;
				this.Type = entry.Type;
				this.Number = entry.Number;
				this.Graphic = entry.Graphic;
				this.RequireExceptional = reqExceptional;
				this.Material = material;
			}
		}

		public SmallFletcherBOD( int amountCur, int amountMax, Type type, int number, int graphic, bool reqExceptional, BulkMaterialType mat )
		{
			this.Hue = 2213;
			this.AmountMax = amountMax;
			this.AmountCur = amountCur;
			this.Type = type;
			this.Number = number;
			this.Graphic = graphic;
			this.RequireExceptional = reqExceptional;
			this.Material = mat;
		}

		public SmallFletcherBOD( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}
}