using System;
using System.Collections;

namespace Server.Items
{
	public enum CraftResource
	{
		None = 0,
		Iron = 1,
		DullCopper,
		ShadowIron,
		Copper,
		Bronze,
		Gold,
		Agapite,
		Verite,
		Valorite,
                Mythril,

		RegularLeather = 101,
		SpinedLeather,
		HornedLeather,
		BarbedLeather,

		RedScales = 201,
		YellowScales,
		BlackScales,
		GreenScales,
		WhiteScales,
		BlueScales,
                CopperScales,
                BronzeScales,
                SilverScales,
                GoldScales,
                OphidianScales,

                Wood = 301,
                Pine,
		Walnut,
		Maple,
		Mahogany,
		Ash,
		Cedar,
		Sycamore,
		Cherry,
                Yew
	}

	public enum CraftResourceType
	{
		None,
		Metal,
		Leather,
		Scales,
                Wood
        }

	public class CraftAttributeInfo
	{
		private int m_WeaponFireDamage;
		private int m_WeaponColdDamage;
		private int m_WeaponPoisonDamage;
		private int m_WeaponEnergyDamage;
		private int m_WeaponDurability;
		private int m_WeaponLuck;
		private int m_WeaponGoldIncrease;
		private int m_WeaponLowerRequirements;

		private int m_ArmorPhysicalResist;
		private int m_ArmorFireResist;
		private int m_ArmorColdResist;
		private int m_ArmorPoisonResist;
		private int m_ArmorEnergyResist;
		private int m_ArmorDurability;
		private int m_ArmorLuck;
		private int m_ArmorGoldIncrease;
		private int m_ArmorLowerRequirements;

		private int m_RunicMinAttributes;
		private int m_RunicMaxAttributes;
		private int m_RunicMinIntensity;
		private int m_RunicMaxIntensity;

		public int WeaponFireDamage{ get{ return m_WeaponFireDamage; } set{ m_WeaponFireDamage = value; } }
		public int WeaponColdDamage{ get{ return m_WeaponColdDamage; } set{ m_WeaponColdDamage = value; } }
		public int WeaponPoisonDamage{ get{ return m_WeaponPoisonDamage; } set{ m_WeaponPoisonDamage = value; } }
		public int WeaponEnergyDamage{ get{ return m_WeaponEnergyDamage; } set{ m_WeaponEnergyDamage = value; } }
		public int WeaponDurability{ get{ return m_WeaponDurability; } set{ m_WeaponDurability = value; } }
		public int WeaponLuck{ get{ return m_WeaponLuck; } set{ m_WeaponLuck = value; } }
		public int WeaponGoldIncrease{ get{ return m_WeaponGoldIncrease; } set{ m_WeaponGoldIncrease = value; } }
		public int WeaponLowerRequirements{ get{ return m_WeaponLowerRequirements; } set{ m_WeaponLowerRequirements = value; } }

		public int ArmorPhysicalResist{ get{ return m_ArmorPhysicalResist; } set{ m_ArmorPhysicalResist = value; } }
		public int ArmorFireResist{ get{ return m_ArmorFireResist; } set{ m_ArmorFireResist = value; } }
		public int ArmorColdResist{ get{ return m_ArmorColdResist; } set{ m_ArmorColdResist = value; } }
		public int ArmorPoisonResist{ get{ return m_ArmorPoisonResist; } set{ m_ArmorPoisonResist = value; } }
		public int ArmorEnergyResist{ get{ return m_ArmorEnergyResist; } set{ m_ArmorEnergyResist = value; } }
		public int ArmorDurability{ get{ return m_ArmorDurability; } set{ m_ArmorDurability = value; } }
		public int ArmorLuck{ get{ return m_ArmorLuck; } set{ m_ArmorLuck = value; } }
		public int ArmorGoldIncrease{ get{ return m_ArmorGoldIncrease; } set{ m_ArmorGoldIncrease = value; } }
		public int ArmorLowerRequirements{ get{ return m_ArmorLowerRequirements; } set{ m_ArmorLowerRequirements = value; } }

		public int RunicMinAttributes{ get{ return m_RunicMinAttributes; } set{ m_RunicMinAttributes = value; } }
		public int RunicMaxAttributes{ get{ return m_RunicMaxAttributes; } set{ m_RunicMaxAttributes = value; } }
		public int RunicMinIntensity{ get{ return m_RunicMinIntensity; } set{ m_RunicMinIntensity = value; } }
		public int RunicMaxIntensity{ get{ return m_RunicMaxIntensity; } set{ m_RunicMaxIntensity = value; } }

		public CraftAttributeInfo()
		{
		}

		public static readonly CraftAttributeInfo Blank;
		public static readonly CraftAttributeInfo DullCopper, ShadowIron, Copper, Bronze, Golden, Agapite, Verite, Valorite, Mythril;
		public static readonly CraftAttributeInfo Spined, Horned, Barbed;
		public static readonly CraftAttributeInfo RedScales, YellowScales, BlackScales, GreenScales, WhiteScales, BlueScales, CopperScales, BronzeScales, SilverScales, GoldScales, OphidianScales;
                public static readonly CraftAttributeInfo Pine, Walnut, Maple, Mahogany, Ash, Cedar, Sycamore, Cherry, Yew;             

		static CraftAttributeInfo()
		{
			Blank = new CraftAttributeInfo();

			CraftAttributeInfo dullCopper = DullCopper = new CraftAttributeInfo();

			dullCopper.ArmorPhysicalResist = 6;
			dullCopper.ArmorDurability = 50;
			dullCopper.ArmorLowerRequirements = 20;
			dullCopper.WeaponDurability = 100;
			dullCopper.WeaponLowerRequirements = 50;
			dullCopper.RunicMinAttributes = 1;
			dullCopper.RunicMaxAttributes = 2;
			dullCopper.RunicMinIntensity = 10;
			dullCopper.RunicMaxIntensity = 35;

			CraftAttributeInfo shadowIron = ShadowIron = new CraftAttributeInfo();

			shadowIron.ArmorPhysicalResist = 2;
			shadowIron.ArmorFireResist = 1;
			shadowIron.ArmorEnergyResist = 5;
			shadowIron.ArmorDurability = 100;
			shadowIron.WeaponColdDamage = 20;
			shadowIron.WeaponDurability = 50;
			shadowIron.RunicMinAttributes = 2;
			shadowIron.RunicMaxAttributes = 2;
			shadowIron.RunicMinIntensity = 20;
			shadowIron.RunicMaxIntensity = 45;

			CraftAttributeInfo copper = Copper = new CraftAttributeInfo();

			copper.ArmorPhysicalResist = 1;
			copper.ArmorFireResist = 1;
			copper.ArmorPoisonResist = 5;
			copper.ArmorEnergyResist = 2;
			copper.WeaponPoisonDamage = 10;
			copper.WeaponEnergyDamage = 20;
			copper.RunicMinAttributes = 2;
			copper.RunicMaxAttributes = 3;
			copper.RunicMinIntensity = 25;
			copper.RunicMaxIntensity = 50;

			CraftAttributeInfo bronze = Bronze = new CraftAttributeInfo();

			bronze.ArmorPhysicalResist = 3;
			bronze.ArmorColdResist = 5;
			bronze.ArmorPoisonResist = 1;
			bronze.ArmorEnergyResist = 1;
			bronze.WeaponFireDamage = 40;
			bronze.RunicMinAttributes = 3;
			bronze.RunicMaxAttributes = 3;
			bronze.RunicMinIntensity = 30;
			bronze.RunicMaxIntensity = 65;

			CraftAttributeInfo golden = Golden = new CraftAttributeInfo();

			golden.ArmorPhysicalResist = 1;
			golden.ArmorFireResist = 1;
			golden.ArmorColdResist = 2;
			golden.ArmorEnergyResist = 2;
			golden.ArmorLuck = 40;
			golden.ArmorLowerRequirements = 30;
			golden.WeaponLuck = 40;
			golden.WeaponLowerRequirements = 50;
			golden.RunicMinAttributes = 3;
			golden.RunicMaxAttributes = 4;
			golden.RunicMinIntensity = 35;
			golden.RunicMaxIntensity = 75;

			CraftAttributeInfo agapite = Agapite = new CraftAttributeInfo();

			agapite.ArmorPhysicalResist = 2;
			agapite.ArmorFireResist = 3;
			agapite.ArmorColdResist = 2;
			agapite.ArmorPoisonResist = 2;
			agapite.ArmorEnergyResist = 2;
			agapite.WeaponColdDamage = 30;
			agapite.WeaponEnergyDamage = 20;
			agapite.RunicMinAttributes = 4;
			agapite.RunicMaxAttributes = 4;
			agapite.RunicMinIntensity = 40;
			agapite.RunicMaxIntensity = 80;

			CraftAttributeInfo verite = Verite = new CraftAttributeInfo();

			verite.ArmorPhysicalResist = 3;
			verite.ArmorFireResist = 3;
			verite.ArmorColdResist = 2;
			verite.ArmorPoisonResist = 3;
			verite.ArmorEnergyResist = 1;
			verite.WeaponPoisonDamage = 40;
			verite.WeaponEnergyDamage = 20;
			verite.RunicMinAttributes = 4;
			verite.RunicMaxAttributes = 5;
			verite.RunicMinIntensity = 45;
			verite.RunicMaxIntensity = 90;

			CraftAttributeInfo valorite = Valorite = new CraftAttributeInfo();

			valorite.ArmorPhysicalResist = 4;
			valorite.ArmorColdResist = 3;
			valorite.ArmorPoisonResist = 3;
			valorite.ArmorEnergyResist = 3;
			valorite.ArmorDurability = 50;
			valorite.WeaponFireDamage = 10;
			valorite.WeaponColdDamage = 20;
			valorite.WeaponPoisonDamage = 10;
			valorite.WeaponEnergyDamage = 20;
			valorite.RunicMinAttributes = 5;
			valorite.RunicMaxAttributes = 5;
			valorite.RunicMinIntensity = 50;
			valorite.RunicMaxIntensity = 100;

                        CraftAttributeInfo mythril = Mythril = new CraftAttributeInfo();

			mythril.ArmorPhysicalResist = 6;
			mythril.ArmorColdResist = 6;
			mythril.ArmorPoisonResist = 6;
			mythril.ArmorEnergyResist = 6;
			mythril.ArmorDurability = 450;
                        mythril.ArmorLowerRequirements = 70;
                        mythril.ArmorGoldIncrease = 100;
			mythril.WeaponColdDamage = 100;
			mythril.RunicMinAttributes = 9;
			mythril.RunicMaxAttributes = 10;
			mythril.RunicMinIntensity = 290;
			mythril.RunicMaxIntensity = 300;

			CraftAttributeInfo spined = Spined = new CraftAttributeInfo();

			spined.ArmorPhysicalResist = 5;
			spined.ArmorLuck = 40;
			spined.RunicMinAttributes = 1;
			spined.RunicMaxAttributes = 3;
			spined.RunicMinIntensity = 20;
			spined.RunicMaxIntensity = 40;

			CraftAttributeInfo horned = Horned = new CraftAttributeInfo();

			horned.ArmorPhysicalResist = 2;
			horned.ArmorFireResist = 3;
			horned.ArmorColdResist = 2;
			horned.ArmorPoisonResist = 2;
			horned.ArmorEnergyResist = 2;
			horned.RunicMinAttributes = 3;
			horned.RunicMaxAttributes = 4;
			horned.RunicMinIntensity = 30;
			horned.RunicMaxIntensity = 70;

			CraftAttributeInfo barbed = Barbed = new CraftAttributeInfo();

			barbed.ArmorPhysicalResist = 2;
			barbed.ArmorFireResist = 1;
			barbed.ArmorColdResist = 2;
			barbed.ArmorPoisonResist = 3;
			barbed.ArmorEnergyResist = 4;
			barbed.RunicMinAttributes = 4;
			barbed.RunicMaxAttributes = 5;
			barbed.RunicMinIntensity = 40;
			barbed.RunicMaxIntensity = 100;

			CraftAttributeInfo red = RedScales = new CraftAttributeInfo();

			red.ArmorPhysicalResist = 7;
			red.ArmorPoisonResist = 8;
                        red.ArmorEnergyResist = 10;
                        red.ArmorFireResist = 10;
                        red.ArmorColdResist = 2;


			CraftAttributeInfo yellow = YellowScales = new CraftAttributeInfo();

			yellow.ArmorPhysicalResist = 3;
			yellow.ArmorLuck = 40;

			CraftAttributeInfo black = BlackScales = new CraftAttributeInfo();

			black.ArmorPhysicalResist = 3;
			black.ArmorColdResist = 10;
                        black.ArmorEnergyResist = 7;

			CraftAttributeInfo green = GreenScales = new CraftAttributeInfo();

			green.ArmorPhysicalResist = 3;
			green.ArmorPoisonResist = 10;
                        green.ArmorEnergyResist = 7;

			CraftAttributeInfo white = WhiteScales = new CraftAttributeInfo();

			white.ArmorPhysicalResist = 3;
			white.ArmorColdResist = 10;
                        white.ArmorEnergyResist = 7;
                        white.ArmorFireResist = 6;

			CraftAttributeInfo blue = BlueScales = new CraftAttributeInfo();

			blue.ArmorPhysicalResist = 3;
			blue.ArmorPoisonResist = 5;
                        blue.ArmorEnergyResist = 12;
                       
                        CraftAttributeInfo coppers = CopperScales = new CraftAttributeInfo();

			coppers.ArmorPhysicalResist = 3;
			coppers.ArmorColdResist = 10;
                        coppers.ArmorEnergyResist = 7;

                        CraftAttributeInfo bronzes = BronzeScales = new CraftAttributeInfo();

			bronzes.ArmorPhysicalResist = 3;
			bronzes.ArmorPoisonResist = 10;
                        bronzes.ArmorEnergyResist = 7;

                        CraftAttributeInfo silver = SilverScales = new CraftAttributeInfo();

			silver.ArmorPhysicalResist = 3;
			silver.ArmorColdResist = 10;
                        silver.ArmorEnergyResist = 7;
                        silver.ArmorFireResist = 6;

                        CraftAttributeInfo gold = GoldScales = new CraftAttributeInfo();

			gold.ArmorPhysicalResist = 10;
			gold.ArmorPoisonResist = 10;
                        gold.ArmorEnergyResist = 10;
                        gold.ArmorFireResist = 10;
                        gold.ArmorColdResist = 10;

                        CraftAttributeInfo ophid = OphidianScales = new CraftAttributeInfo();

			ophid.ArmorPhysicalResist = 0;
			ophid.ArmorPoisonResist = 0;
                        ophid.ArmorEnergyResist = 0;
                        ophid.ArmorFireResist = 0;
                        ophid.ArmorColdResist = 0;

                        CraftAttributeInfo pine = Pine = new CraftAttributeInfo();

                        pine.ArmorPhysicalResist = 6;
			pine.ArmorDurability = 50;
			pine.ArmorLowerRequirements = 20;
			pine.WeaponDurability = 100;
			pine.WeaponLowerRequirements = 50;
			pine.RunicMinAttributes = 1;
			pine.RunicMaxAttributes = 2;
			pine.RunicMinIntensity = 10;
			pine.RunicMaxIntensity = 35;

                        CraftAttributeInfo walnut = Walnut = new CraftAttributeInfo();

                        walnut.ArmorPhysicalResist = 2;
			walnut.ArmorFireResist = 1;
			walnut.ArmorEnergyResist = 5;
			walnut.ArmorDurability = 100;
			walnut.WeaponColdDamage = 20;
			walnut.WeaponDurability = 50;
			walnut.RunicMinAttributes = 2;
			walnut.RunicMaxAttributes = 2;
			walnut.RunicMinIntensity = 20;
			walnut.RunicMaxIntensity = 45;

			CraftAttributeInfo maple = Maple = new CraftAttributeInfo();

			maple.ArmorPhysicalResist = 1;
			maple.ArmorFireResist = 1;
			maple.ArmorPoisonResist = 5;
			maple.ArmorEnergyResist = 2;
			maple.WeaponPoisonDamage = 10;
			maple.WeaponEnergyDamage = 20;
			maple.RunicMinAttributes = 2;
			maple.RunicMaxAttributes = 3;
			maple.RunicMinIntensity = 25;
			maple.RunicMaxIntensity = 50;

			CraftAttributeInfo mahogany = Mahogany = new CraftAttributeInfo();

			mahogany.ArmorPhysicalResist = 3;
			mahogany.ArmorColdResist = 5;
			mahogany.ArmorPoisonResist = 1;
			mahogany.ArmorEnergyResist = 1;
			mahogany.WeaponFireDamage = 40;
			mahogany.RunicMinAttributes = 3;
			mahogany.RunicMaxAttributes = 3;
			mahogany.RunicMinIntensity = 30;
			mahogany.RunicMaxIntensity = 65;


			CraftAttributeInfo ash = Ash = new CraftAttributeInfo();

			ash.ArmorPhysicalResist = 1;
			ash.ArmorFireResist = 1;
			ash.ArmorColdResist = 2;
			ash.ArmorEnergyResist = 2;
			ash.ArmorLuck = 40;
			ash.ArmorLowerRequirements = 30;
			ash.WeaponLuck = 40;
			ash.WeaponLowerRequirements = 50;
			ash.RunicMinAttributes = 3;
			ash.RunicMaxAttributes = 4;
			ash.RunicMinIntensity = 35;
			ash.RunicMaxIntensity = 75;

			CraftAttributeInfo cedar = Cedar = new CraftAttributeInfo();

			cedar.ArmorPhysicalResist = 2;
			cedar.ArmorFireResist = 3;
			cedar.ArmorColdResist = 2;
			cedar.ArmorPoisonResist = 2;
			cedar.ArmorEnergyResist = 2;
			cedar.WeaponColdDamage = 30;
			cedar.WeaponEnergyDamage = 20;
			cedar.RunicMinAttributes = 4;
			cedar.RunicMaxAttributes = 4;
			cedar.RunicMinIntensity = 40;
			cedar.RunicMaxIntensity = 80;

			CraftAttributeInfo sycamore = Sycamore = new CraftAttributeInfo();

			sycamore.ArmorPhysicalResist = 3;
			sycamore.ArmorFireResist = 3;
			sycamore.ArmorColdResist = 2;
			sycamore.ArmorPoisonResist = 3;
			sycamore.ArmorEnergyResist = 1;
			sycamore.WeaponPoisonDamage = 40;
			sycamore.WeaponEnergyDamage = 20;
			sycamore.RunicMinAttributes = 4;
			sycamore.RunicMaxAttributes = 5;
			sycamore.RunicMinIntensity = 45;
			sycamore.RunicMaxIntensity = 90;

			CraftAttributeInfo cherry = Cherry = new CraftAttributeInfo();

			cherry.ArmorPhysicalResist = 4;
			cherry.ArmorColdResist = 3;
			cherry.ArmorPoisonResist = 3;
			cherry.ArmorEnergyResist = 3;
			cherry.ArmorDurability = 50;
			cherry.WeaponFireDamage = 10;
			cherry.WeaponColdDamage = 20;
			cherry.WeaponPoisonDamage = 10;
			cherry.WeaponEnergyDamage = 20;
			cherry.RunicMinAttributes = 5;
			cherry.RunicMaxAttributes = 5;
			cherry.RunicMinIntensity = 50;
			cherry.RunicMaxIntensity = 100;

                        CraftAttributeInfo yew = Yew = new CraftAttributeInfo();

			yew.ArmorPhysicalResist = 6;
			yew.ArmorColdResist = 6;
			yew.ArmorPoisonResist = 6;
			yew.ArmorEnergyResist = 6;
			yew.ArmorDurability = 450;
                        yew.ArmorLowerRequirements = 70;
                        yew.ArmorGoldIncrease = 100;
			yew.WeaponColdDamage = 100;
			yew.RunicMinAttributes = 9;
			yew.RunicMaxAttributes = 10;
			yew.RunicMinIntensity = 290;
			yew.RunicMaxIntensity = 300;


		}
	}

	public class CraftResourceInfo
	{
		private int m_Hue;
		private int m_Number;
		private string m_Name;
		private CraftAttributeInfo m_AttributeInfo;
		private CraftResource m_Resource;
		private Type[] m_ResourceTypes;

		public int Hue{ get{ return m_Hue; } }
		public int Number{ get{ return m_Number; } }
		public string Name{ get{ return m_Name; } }
		public CraftAttributeInfo AttributeInfo{ get{ return m_AttributeInfo; } }
		public CraftResource Resource{ get{ return m_Resource; } }
		public Type[] ResourceTypes{ get{ return m_ResourceTypes; } }

		public CraftResourceInfo( int hue, int number, string name, CraftAttributeInfo attributeInfo, CraftResource resource, params Type[] resourceTypes )
		{
			m_Hue = hue;
			m_Number = number;
			m_Name = name;
			m_AttributeInfo = attributeInfo;
			m_Resource = resource;
			m_ResourceTypes = resourceTypes;

			for ( int i = 0; i < resourceTypes.Length; ++i )
				CraftResources.RegisterType( resourceTypes[i], resource );
		}
	}

	public class CraftResources
	{
		private static CraftResourceInfo[] m_MetalInfo = new CraftResourceInfo[]
			{
				new CraftResourceInfo( 0x000, 1053109, "Iron",			CraftAttributeInfo.Blank,		CraftResource.Iron,			typeof( IronIngot ),		typeof( IronOre ),		typeof( Granite ) ),
				new CraftResourceInfo( 0x973, 1053108, "Dull Copper",	        CraftAttributeInfo.DullCopper,	        CraftResource.DullCopper,		typeof( DullCopperIngot ),	typeof( DullCopperOre ),	typeof( DullCopperGranite ) ),
				new CraftResourceInfo( 0x966, 1053107, "Shadow Iron",	        CraftAttributeInfo.ShadowIron,	        CraftResource.ShadowIron,		typeof( ShadowIronIngot ),	typeof( ShadowIronOre ),	typeof( ShadowIronGranite ) ),
				new CraftResourceInfo( 0x96D, 1053106, "Copper",		CraftAttributeInfo.Copper,		CraftResource.Copper,			typeof( CopperIngot ),		typeof( CopperOre ),		typeof( CopperGranite ) ),
				new CraftResourceInfo( 0x972, 1053105, "Bronze",		CraftAttributeInfo.Bronze,		CraftResource.Bronze,			typeof( BronzeIngot ),		typeof( BronzeOre ),		typeof( BronzeGranite ) ),
				new CraftResourceInfo( 0x8A5, 1053104, "Gold",			CraftAttributeInfo.Golden,		CraftResource.Gold,			typeof( GoldIngot ),		typeof( GoldOre ),		typeof( GoldGranite ) ),
				new CraftResourceInfo( 0x979, 1053103, "Agapite",		CraftAttributeInfo.Agapite,		CraftResource.Agapite,			typeof( AgapiteIngot ),		typeof( AgapiteOre ),		typeof( AgapiteGranite ) ),
				new CraftResourceInfo( 0x89F, 1053102, "Verite",		CraftAttributeInfo.Verite,		CraftResource.Verite,			typeof( VeriteIngot ),		typeof( VeriteOre ),		typeof( VeriteGranite ) ),
				new CraftResourceInfo( 0x8AB, 1053101, "Valorite",		CraftAttributeInfo.Valorite,	        CraftResource.Valorite,			typeof( ValoriteIngot ),	typeof( ValoriteOre ),		typeof( ValoriteGranite ) ),
                                new CraftResourceInfo( 1171, 0, "Mythril",		        CraftAttributeInfo.Mythril,	        CraftResource.Mythril,			typeof( MythrilIngot ),	        typeof( MythrilOre ),		typeof( ValoriteGranite ) ),
                        };
               
                private static CraftResourceInfo[] m_WoodInfo = new CraftResourceInfo[]
                        {
                                new CraftResourceInfo( 0x000, 0, "Wood",	CraftAttributeInfo.Blank,	CraftResource.Wood,		typeof( Board ),                 typeof( Log ) ),
                                new CraftResourceInfo( 2108, 0, "Pine",	        CraftAttributeInfo.Pine,	CraftResource.Pine,		typeof( PineBoard ),             typeof( PineLog ) ),
				new CraftResourceInfo( 1747, 0, "Walnut",	CraftAttributeInfo.Walnut,	CraftResource.Walnut,		typeof( WalnutBoard ),           typeof( WalnutLog ) ),
				new CraftResourceInfo( 2418, 0, "Maple",	CraftAttributeInfo.Maple,	CraftResource.Maple,		typeof( MapleBoard ),            typeof( MapleLog ) ),
				new CraftResourceInfo( 2315, 0, "Mahogany",	CraftAttributeInfo.Mahogany,	CraftResource.Mahogany,		typeof( MahoganyBoard ),         typeof( MahoganyLog ) ),
				new CraftResourceInfo( 1448, 0, "Ash",	        CraftAttributeInfo.Ash,		CraftResource.Ash,		typeof( AshBoard ),              typeof( AshLog ) ),
				new CraftResourceInfo( 2307, 0, "Cedar",	CraftAttributeInfo.Cedar,	CraftResource.Cedar,		typeof( CedarBoard ),            typeof( CedarLog ) ),
                                new CraftResourceInfo( 1803, 0, "Sycamore",	CraftAttributeInfo.Sycamore,	CraftResource.Sycamore,		typeof( SycamoreBoard ),         typeof( SycamoreLog ) ),
				new CraftResourceInfo( 1517, 0, "Cherry",	CraftAttributeInfo.Cherry,	CraftResource.Cherry,		typeof( CherryBoard ),           typeof( CherryLog ) ),
				new CraftResourceInfo( 1808, 0, "Yew",	        CraftAttributeInfo.Yew,		CraftResource.Yew,		typeof( YewBoard ),              typeof( YewLog ) )
                        };
                        
		private static CraftResourceInfo[] m_ScaleInfo = new CraftResourceInfo[]
			{
				new CraftResourceInfo( 0x66D, 1053129, "Red Scales",	CraftAttributeInfo.RedScales,		CraftResource.RedScales,		typeof( RedScales ) ),
				new CraftResourceInfo( 2213, 1053130, "Yellow Scales",	CraftAttributeInfo.YellowScales,	CraftResource.YellowScales,		typeof( YellowScales ) ),
				new CraftResourceInfo( 0x455, 1053131, "Black Scales",	CraftAttributeInfo.BlackScales,		CraftResource.BlackScales,		typeof( BlackScales ) ),
				new CraftResourceInfo( 0x851, 1053132, "Green Scales",	CraftAttributeInfo.GreenScales,		CraftResource.GreenScales,		typeof( GreenScales ) ),
				new CraftResourceInfo( 0x8FD, 1053133, "White Scales",	CraftAttributeInfo.WhiteScales,		CraftResource.WhiteScales,		typeof( WhiteScales ) ),
				new CraftResourceInfo( 0x8B0, 1053134, "Blue Scales",	CraftAttributeInfo.BlueScales,		CraftResource.BlueScales,		typeof( BlueScales ) ),
                                new CraftResourceInfo( 1875, 0, "Copper Scales",	CraftAttributeInfo.CopperScales,	CraftResource.CopperScales,	        typeof( CopperScales ) ),
				new CraftResourceInfo( 1704, 0, "Bronze Scales",	CraftAttributeInfo.BronzeScales,	CraftResource.BronzeScales,	        typeof( BronzeScales ) ),
				new CraftResourceInfo( 1154, 0, "Silver Scales",	CraftAttributeInfo.SilverScales,	CraftResource.SilverScales,	        typeof( SilverScales ) ),
				new CraftResourceInfo( 0x8A8, 0, "Gold Scales",	        CraftAttributeInfo.GoldScales,		CraftResource.GoldScales,		typeof( GoldScales ) ),
                                new CraftResourceInfo( 2108, 0, "Ophidian Scales",	CraftAttributeInfo.OphidianScales,	CraftResource.OphidianScales,		typeof( OphidianScales ) )
			};
               
               
               	private static CraftResourceInfo[] m_LeatherInfo = new CraftResourceInfo[]
			{
				new CraftResourceInfo( 0x000, 1049353, "Normal",		CraftAttributeInfo.Blank,		CraftResource.RegularLeather,	typeof( Leather ),			typeof( Hides ) ),
				new CraftResourceInfo( 0x283, 1049354, "Spined",		CraftAttributeInfo.Spined,		CraftResource.SpinedLeather,	typeof( SpinedLeather ),	typeof( SpinedHides ) ),
				new CraftResourceInfo( 0x227, 1049355, "Horned",		CraftAttributeInfo.Horned,		CraftResource.HornedLeather,	typeof( HornedLeather ),	typeof( HornedHides ) ),
				new CraftResourceInfo( 0x1C1, 1049356, "Barbed",		CraftAttributeInfo.Barbed,		CraftResource.BarbedLeather,	typeof( BarbedLeather ),	typeof( BarbedHides ) )
		};

		private static CraftResourceInfo[] m_AOSLeatherInfo = new CraftResourceInfo[]
			{
				new CraftResourceInfo( 0x000, 1049353, "Normal",		CraftAttributeInfo.Blank,		CraftResource.RegularLeather,	typeof( Leather ),			typeof( Hides ) ),
				new CraftResourceInfo( 0x8AC, 1049354, "Spined",		CraftAttributeInfo.Spined,		CraftResource.SpinedLeather,	typeof( SpinedLeather ),	typeof( SpinedHides ) ),
				new CraftResourceInfo( 0x845, 1049355, "Horned",		CraftAttributeInfo.Horned,		CraftResource.HornedLeather,	typeof( HornedLeather ),	typeof( HornedHides ) ),
				new CraftResourceInfo( 0x851, 1049356, "Barbed",		CraftAttributeInfo.Barbed,		CraftResource.BarbedLeather,	typeof( BarbedLeather ),	typeof( BarbedHides ) ),
		};
         
                                
		/// <summary>
		/// Returns true if '<paramref name="resource"/>' is None, Iron, or RegularLeather, or Plain. False if otherwise.
		/// </summary>
		public static bool IsStandard( CraftResource resource )
		{
			return ( resource == CraftResource.None || resource == CraftResource.Iron || resource == CraftResource.RegularLeather || resource == CraftResource.Wood );
		}

		private static Hashtable m_TypeTable;

		/// <summary>
		/// Registers that '<paramref name="resourceType"/>' uses '<paramref name="resource"/>' so that it can later be queried by <see cref="CraftResources.GetFromType"/>
		/// </summary>
		public static void RegisterType( Type resourceType, CraftResource resource )
		{
			if ( m_TypeTable == null )
				m_TypeTable = new Hashtable();

			m_TypeTable[resourceType] = resource;
		}

		/// <summary>
		/// Returns the <see cref="CraftResource"/> value for which '<paramref name="resourceType"/>' uses -or- CraftResource.None if an unregistered type was specified.
		/// </summary>
		public static CraftResource GetFromType( Type resourceType )
		{
			if ( m_TypeTable == null )
				return CraftResource.None;

			object obj = m_TypeTable[resourceType];

			if ( !(obj is CraftResource) )
				return CraftResource.None;

			return (CraftResource)obj;
		}

		/// <summary>
		/// Returns a <see cref="CraftResourceInfo"/> instance describing '<paramref name="resource"/>' -or- null if an invalid resource was specified.
		/// </summary>
		public static CraftResourceInfo GetInfo( CraftResource resource )
		{
			CraftResourceInfo[] list = null;

			switch ( GetType( resource ) )
			{
				case CraftResourceType.Metal: list = m_MetalInfo; break;
				case CraftResourceType.Leather: list = Core.AOS ? m_AOSLeatherInfo : m_LeatherInfo; break;
				case CraftResourceType.Scales: list = m_ScaleInfo; break;
                                case CraftResourceType.Wood: list = m_WoodInfo; break;
                        }

			if ( list != null )
			{
				int index = GetIndex( resource );

				if ( index >= 0 && index < list.Length )
					return list[index];
			}

			return null;
		}

		/// <summary>
		/// Returns a <see cref="CraftResourceType"/> value indiciating the type of '<paramref name="resource"/>'.
		/// </summary>
		public static CraftResourceType GetType( CraftResource resource )
		{
			if ( resource >= CraftResource.Iron && resource <= CraftResource.Mythril )
				return CraftResourceType.Metal;
                        
                        if ( resource >= CraftResource.Wood && resource <= CraftResource.Yew )
				return CraftResourceType.Wood;

			if ( resource >= CraftResource.RegularLeather && resource <= CraftResource.BarbedLeather )
				return CraftResourceType.Leather;

			if ( resource >= CraftResource.RedScales && resource <= CraftResource.OphidianScales )
				return CraftResourceType.Scales;
                                                

			return CraftResourceType.None;
		}

		/// <summary>
		/// Returns the first <see cref="CraftResource"/> in the series of resources for which '<paramref name="resource"/>' belongs.
		/// </summary>
		public static CraftResource GetStart( CraftResource resource )
		{
			switch ( GetType( resource ) )
			{
				case CraftResourceType.Metal: return CraftResource.Iron;
				case CraftResourceType.Leather: return CraftResource.RegularLeather;
				case CraftResourceType.Scales: return CraftResource.RedScales;
                                case CraftResourceType.Wood: return CraftResource.Wood;
                        }

			return CraftResource.None;
		}

		/// <summary>
		/// Returns the index of '<paramref name="resource"/>' in the seriest of resources for which it belongs.
		/// </summary>
		public static int GetIndex( CraftResource resource )
		{
			CraftResource start = GetStart( resource );

			if ( start == CraftResource.None )
				return 0;

			return (int)(resource - start);
		}

		/// <summary>
		/// Returns the <see cref="CraftResourceInfo.Number"/> property of '<paramref name="resource"/>' -or- 0 if an invalid resource was specified.
		/// </summary>
		public static int GetLocalizationNumber( CraftResource resource )
		{
			CraftResourceInfo info = GetInfo( resource );

			return ( info == null ? 0 : info.Number );
		}

		/// <summary>
		/// Returns the <see cref="CraftResourceInfo.Hue"/> property of '<paramref name="resource"/>' -or- 0 if an invalid resource was specified.
		/// </summary>
		public static int GetHue( CraftResource resource )
		{
			CraftResourceInfo info = GetInfo( resource );

			return ( info == null ? 0 : info.Hue );
		}

		/// <summary>
		/// Returns the <see cref="CraftResourceInfo.Name"/> property of '<paramref name="resource"/>' -or- an empty string if the resource specified was invalid.
		/// </summary>
		public static string GetName( CraftResource resource )
		{
			CraftResourceInfo info = GetInfo( resource );

			return ( info == null ? String.Empty : info.Name );
		}

		/// <summary>
		/// Returns the <see cref="CraftResource"/> value which represents '<paramref name="info"/>' -or- CraftResource.None if unable to convert.
		/// </summary>
		public static CraftResource GetFromOreInfo( OreInfo info )
		{
			if ( info.Name.IndexOf( "Spined" ) >= 0 )
				return CraftResource.SpinedLeather;
			else if ( info.Name.IndexOf( "Horned" ) >= 0 )
				return CraftResource.HornedLeather;
			else if ( info.Name.IndexOf( "Barbed" ) >= 0 )
				return CraftResource.BarbedLeather;
			else if ( info.Name.IndexOf( "Leather" ) >= 0 )
				return CraftResource.RegularLeather;

			if ( info.Level == 0 )
				return CraftResource.Iron;
			else if ( info.Level == 1 )
				return CraftResource.DullCopper;
			else if ( info.Level == 2 )
				return CraftResource.ShadowIron;
			else if ( info.Level == 3 )
				return CraftResource.Copper;
			else if ( info.Level == 4 )
				return CraftResource.Bronze;
			else if ( info.Level == 5 )
				return CraftResource.Gold;
			else if ( info.Level == 6 )
				return CraftResource.Agapite;
			else if ( info.Level == 7 )
				return CraftResource.Verite;
			else if ( info.Level == 8 )
				return CraftResource.Valorite;
                        else if ( info.Level == 9 )
				return CraftResource.Mythril;
                        else if ( info.Level == 10 )
				return CraftResource.Wood;
                        else if ( info.Level == 11 )
				return CraftResource.Pine;
			else if ( info.Level == 12 )
				return CraftResource.Walnut;
			else if ( info.Level == 13 )
				return CraftResource.Maple;
			else if ( info.Level == 14 )
				return CraftResource.Mahogany;
			else if ( info.Level == 15 )
				return CraftResource.Ash;
			else if ( info.Level == 16 )
				return CraftResource.Cedar;
			else if ( info.Level == 17 )
				return CraftResource.Sycamore;
                        else if ( info.Level == 18 )
				return CraftResource.Cherry;
                        else if ( info.Level == 19 )
				return CraftResource.Yew;
                        
                                               
			return CraftResource.None;
		}

		/// <summary>
		/// Returns the <see cref="CraftResource"/> value which represents '<paramref name="info"/>', using '<paramref name="material"/>' to help resolve leather OreInfo instances.
		/// </summary>
		public static CraftResource GetFromOreInfo( OreInfo info, ArmorMaterialType material )
		{
			if ( material == ArmorMaterialType.Studded || material == ArmorMaterialType.Leather || material == ArmorMaterialType.Spined ||
				material == ArmorMaterialType.Horned || material == ArmorMaterialType.Barbed )
			{
				if ( info.Level == 0 )
					return CraftResource.RegularLeather;
				else if ( info.Level == 1 )
					return CraftResource.SpinedLeather;
				else if ( info.Level == 2 )
					return CraftResource.HornedLeather;
				else if ( info.Level == 3 )
					return CraftResource.BarbedLeather;

				return CraftResource.None;
			}

			return GetFromOreInfo( info );
		}
	}

	// NOTE: This class is only for compatability with very old RunUO versions.
	// No changes to it should be required for custom resources.
	public class OreInfo
	{
		public static readonly OreInfo Iron		= new OreInfo( 0, 0x000, "Iron" );
		public static readonly OreInfo DullCopper	= new OreInfo( 1, 0x973, "Dull Copper" );
		public static readonly OreInfo ShadowIron	= new OreInfo( 2, 0x966, "Shadow Iron" );
		public static readonly OreInfo Copper		= new OreInfo( 3, 0x96D, "Copper" );
		public static readonly OreInfo Bronze		= new OreInfo( 4, 0x972, "Bronze" );
		public static readonly OreInfo Gold		= new OreInfo( 5, 0x8A5, "Gold" );
		public static readonly OreInfo Agapite		= new OreInfo( 6, 0x979, "Agapite" );
		public static readonly OreInfo Verite		= new OreInfo( 7, 0x89F, "Verite" );
		public static readonly OreInfo Valorite		= new OreInfo( 8, 0x8AB, "Valorite" );
                public static readonly OreInfo Mythril		= new OreInfo( 9, 1171, "Mythril" );
                public static readonly OreInfo Wood	        = new OreInfo( 10, 0, "Wood" );
                public static readonly OreInfo Pine	        = new OreInfo( 11, 2108, "Pine" );
		public static readonly OreInfo Walnut	        = new OreInfo( 12, 1747, "Walnut" );
		public static readonly OreInfo Maple		= new OreInfo( 13, 2418, "Maple" );
		public static readonly OreInfo Mahogany		= new OreInfo( 14, 2315, "Mahogany" );
		public static readonly OreInfo Ash		= new OreInfo( 15, 1448, "Ash" );
		public static readonly OreInfo Cedar		= new OreInfo( 16, 2307, "Cedar" );
		public static readonly OreInfo Sycamore		= new OreInfo( 17, 1803, "Sycamore" );
		public static readonly OreInfo Cherry		= new OreInfo( 18, 1517, "Cherry" );
                public static readonly OreInfo Yew		= new OreInfo( 19, 1808, "Yew" );

		private int m_Level;
		private int m_Hue;
		private string m_Name;

		public OreInfo( int level, int hue, string name )
		{
			m_Level = level;
			m_Hue = hue;
			m_Name = name;
		}

		public int Level
		{
			get
			{
				return m_Level;
			}
		}

		public int Hue
		{
			get
			{
				return m_Hue;
			}
		}

		public string Name
		{
			get
			{
				return m_Name;
			}
		}
	}
}